<?php

declare(strict_types=1);

namespace Drupal\paragraphs_blokkli\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;

/**
 * Configure Paragraph Builder settings for this site.
 */
final class SettingsForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId(): string {
    return 'paragraphs_blokkli_settings';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames(): array {
    return ['paragraphs_blokkli.settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function defaultConfiguration() {
    $default_values = [
      'schema_file' => '',
    ];

    $media_types = \Drupal::entityTypeManager()
      ->getStorage('media_type')
      ->loadMultiple();
    foreach ($media_types as $media_type) {
      $bundle = $media_type->id();

      // For now only remote video and image are supported.
      if (in_array($bundle, ['remote_video', 'image'])) {
        $default_values['media_mapping'][$bundle]['label'] = $media_type->label();
      }
    }

    return $default_values;
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $form['schema_file'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Schema File'),
      '#description' => $this->t('The schema file is generated by the frontend application.'),
      '#default_value' => $this->config('paragraphs_blokkli.settings')->get('schema_file'),
    ];

    $default_values = $this->defaultConfiguration();

    $form['#tree'] = TRUE;

    $media_bundles = [];
    foreach ($default_values['media_mapping'] as $bundle => $info) {
      $media_bundles[$bundle] = $info['label'];
    }

    $paragraph_bundle_info = \Drupal::service('entity_type.bundle.info')->getBundleInfo('paragraph');
    $paragraph_bundles[''] = $this->t('Select a paragraph bundle');
    foreach ($paragraph_bundle_info as $bundle => $info) {
      $paragraph_bundles[$bundle] = $info['label'];
    }

    $saved_values = $this->config('paragraphs_blokkli.settings')->get('media_mapping');
    foreach ($default_values['media_mapping'] as $bundle => $media_type) {
      $form[$bundle] = [
        '#type' => 'details',
        '#title' => $media_type['label'] . ' ' . $this->t('to paragraph field mapping'),
        '#open' => TRUE,
        '#prefix' => '<div id="' . $bundle . '-wrapper">',
        '#suffix' => '</div>',
      ];
      $form[$bundle]['paragraph_bundle'] = [
        '#type' => 'select',
        '#title' => $this->t('Paragraph Bundle'),
        '#options' => $paragraph_bundles,
        '#description' => 'Target paragraph bandle type.',
        '#default_value' => !empty($saved_values[$bundle]['paragraph_bundle']) ? $saved_values[$bundle]['paragraph_bundle'] : '',
        '#maxlength' => 255,
        '#ajax' => [
          'callback' => [$this, 'paragraphFieldsCallback'],
          'disable-refocus' => FALSE,
          'event' => 'change',
          'wrapper' => $bundle . '-wrapper',
          'progress' => [
            'type' => 'throbber',
            'message' => $this->t('Loading fields...'),
          ],
        ],
      ];

      $paragraph_bundle_fields = [];
      $selectedValue = '';
      // || $selectedValue = $form_state->getValue($bundle)['paragraph_bundle']) {
      if (!empty($saved_values[$bundle]['paragraph_bundle'])) {
        $selectedValue = $saved_values[$bundle]['paragraph_bundle'];
      }
      if (!empty($form_state->getValue($bundle)['paragraph_bundle'])) {
        $selectedValue = $form_state->getValue($bundle)['paragraph_bundle'];
      }
      if ($selectedValue) {
        $fields = \Drupal::service('entity_field.manager')->getFieldDefinitions('paragraph', $selectedValue);
        foreach ($fields as $field_name => $field) {
          if ($field->getType() == 'entity_reference' && $field->getSetting('target_type') == 'media') {
            $paragraph_bundle_fields[$field_name] = $field->getLabel();
          }
        }
      }

      $form[$bundle]['paragraph_field'] = [
        '#type' => 'select',
        '#title' => $this->t('Paragraph Field'),
        '#options' => $paragraph_bundle_fields,
        '#description' => 'Target field on the paragraph type.',
        '#default_value' => !empty($paragraph_bundle) ? $paragraph_bundle : NULL,
        '#maxlength' => 255,
        '#chosen' => FALSE,
        '#states' => [
          'visible' => [
            ':input[name="' . $bundle . '[paragraph_bundle]"]' => ['!value' => ''],
          ],
        ],
      ];
    }

    return parent::buildForm($form, $form_state);
  }

  public function validateForm(array &$form, FormStateInterface $form_state) {
    parent::validateForm($form, $form_state);

    $file_path = $form_state->getValue('schema_file');
    if (!empty($file_path) && !file_exists($file_path)) {
      $form_state->setErrorByName('schema_file', $this->t('The file does not exist.'));
    }
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state): void {

    $mapping = [];
    $default_values = $this->defaultConfiguration();
    foreach ($default_values['media_mapping'] as $bundle => $media_type) {
      $mapping[$bundle] = [
        'paragraph_bundle' => $form_state->getValue($bundle)['paragraph_bundle'],
        'paragraph_field' => $form_state->getValue($bundle)['paragraph_field'],
      ];
    }

    $this->config('paragraphs_blokkli.settings')
      ->set('schema_file', $form_state->getValue('schema_file'))
      ->set('media_mapping', $mapping)
      ->save();
    parent::submitForm($form, $form_state);
  }

  public function paragraphFieldsCallback(array &$form, FormStateInterface $form_state) {
    $trigger = $form_state->getTriggeringElement();
    $bundle = $trigger['#value'];
    return $form[$bundle];
  }

}
