<?php

declare(strict_types=1);

namespace Drupal\paragraphs_blokkli;

use Drupal\Component\Datetime\TimeInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Entity\FieldableEntityInterface;
use Drupal\Core\Entity\RevisionableInterface;
use Drupal\Core\Entity\RevisionLogInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList;
use Drupal\field\Entity\FieldConfig;

class ParagraphsBlokkliManager {

  /**
   * The entity type manager.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected EntityTypeManagerInterface $entityTypeManager;

  /**
   * The paragraphs blokkli edit state storage.
   */
  protected ParagraphsBlokkliEditStateStorageInterface $editStateStorage;

  /**
   * The current user.
   */
  protected AccountInterface $currentUser;

  /**
   * The time service.
   *
   * @var \Drupal\Component\Datetime\TimeInterface
   */
  protected TimeInterface $time;

  /**
   * ParagraphsBehaviorManager constructor.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entity_type_manager
   *   The entity type manager service.
   * @param AccountInterface $currentUser
   *   The current user.
   */
  public function __construct(EntityTypeManagerInterface $entity_type_manager, AccountInterface $currentUser, TimeInterface $time) {
    $this->entityTypeManager = $entity_type_manager;
    $this->editStateStorage = $entity_type_manager->getStorage('paragraphs_blokkli_edit_state');
    $this->currentUser = $currentUser;
    $this->time = $time;
  }

  /**
   * Get the edit state.
   */
  public function getParagraphsEditState(FieldableEntityInterface $entity, bool $noCreate = FALSE): ParagraphsBlokkliEditStateInterface|null {
    $state = $this->editStateStorage->loadForEntity($entity);
    if ($state) {
      return $state;
    }

    if ($noCreate) {
      return NULL;
    }

    $state = $this->editStateStorage->create([
      'host_entity_type' => $entity->getEntityTypeId(),
      'host_entity_uuid' => $entity->uuid(),
    ]);
    $state->save();
    return $state;
  }

  /**
   * Get allowed paragraph types for a field.
   *
   * @param \Drupal\entity_reference_revisions\EntityReferenceRevisionsFieldItemList $field
   *   The entity reference revisions field item list.
   *
   * @return \Drupal\paragraphs\ParagraphsTypeInterface[]
   *   The array of allowed paragraph types.
   */
  public function getAllowedParagraphTypes(EntityReferenceRevisionsFieldItemList $field): array {
    // Get the field definition.
    $definition = $field->getFieldDefinition();
    $handlerSettings = $definition->getSetting('handler_settings');

    // Contains an array of bundle IDs.
    $targetBundles = array_keys($handlerSettings['target_bundles'] ?? []);
    if ($definition instanceof FieldConfig) {
      $storage = $definition->getFieldStorageDefinition();
      $targetType = $storage->getSetting('target_type');
      $storage = $this->entityTypeManager->getStorage($targetType);
      $bundleEntityType = $storage->getEntityType()->getBundleEntityType();
      $typeStorage = $this->entityTypeManager->getStorage($bundleEntityType);
      return $typeStorage->loadMultiple($targetBundles);
    }

    return [];
  }

  /**
   * Save the paragraphs blokkli context.
   */
  public function saveState(EditStateInterface $state) {
    $state->save();
  }

  /**
   * Take ownership of an edit state.
   */
  public function takeOwnership(EditStateInterface $state) {
    if ($this->currentUser->hasPermission('take ownership of paragraphs blokkli edit state')) {
      $state->setOwnerId($this->currentUser->id());
    }
  }

  /**
   * Save the paragraphs blokkli context.
   */
  public function revertAllChanges(EditStateInterface $state) {
    $state->delete();
  }

  /**
   * Publish all changes.
   */
  public function publish(EditStateInterface $state, $force_reload = FALSE) {
    $entity = $state->getHostEntity($force_reload);
    $mutatedState = $state->getMutatedState(TRUE);
    $fields = $mutatedState->getFields();

    foreach ($fields as $field) {
      // Update the paragraphs field on the host entity with the mutated paragraphs.
      if (
        $field->getEntityType() === $state->getHostEntityType() &&
        $field->getEntityUuid() === $state->getHostEntityUuid()
      ) {
        $entity->set($field->getFieldName(), $field->getParagraphs());
      }
    }

    if ($entity->getEntityType()->hasKey('revision')) {
      // Revision handling for revisionable entities.
      if ($entity instanceof RevisionableInterface) {
        $entity->setNewRevision(TRUE);
      }
      if ($entity instanceof RevisionLogInterface) {
        $entity->setRevisionUserId($state->getOwnerId());
        $entity->setRevisionCreationTime($this->time->getRequestTime());
      }
    }

    // Save entity.
    $entity->save();

    $state->delete();
  }

}
