<?php

namespace Drupal\paragraphs_blokkli\Plugin\ParagraphsBlokkli\Mutation;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\file\FileRepositoryInterface;
use Drupal\file\FileUsage\FileUsageInterface;
use Drupal\paragraphs_blokkli\ParagraphMutationContextInterface;
use Drupal\paragraphs_blokkli\ParagraphMutationPluginBase;
use Drupal\paragraphs_blokkli\ParagraphProxy;
use Drupal\paragraphs_blokkli\ParagraphsBlokkliHelper;
use Drupal\rokka\RokkaService;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Adds a new image paragraph.
 *
 * @ParagraphMutation(
 *   id = "add_image",
 *   label = @Translation("Add Image"),
 *   description = @Translation("Adds a new paragraph of type image."),
 *   arguments = {
 *     "data" = @ContextDefinition("string",
 *       label = @Translation("The base64 encoded image data."),
 *     ),
 *     "fileName" = @ContextDefinition("string",
 *       label = @Translation("The name of the file."),
 *     ),
 *     "hostType" = @ContextDefinition("string",
 *       label = @Translation("The entity type of the target host.")
 *     ),
 *     "hostUuid" = @ContextDefinition("string",
 *       label = @Translation("The UUID of the target host.")
 *     ),
 *     "hostFieldName" = @ContextDefinition("string",
 *       label = @Translation("The field name of the target host.")
 *     ),
 *     "afterUuid" = @ContextDefinition("string",
 *       label = @Translation("The UUID of the paragraph after which to add this one."),
 *       required = FALSE,
 *     ),
 *   }
 * )
 */
class AddImage extends ParagraphMutationPluginBase {

  /**
   * The field mapping.
   *
   * @var array
   */
  protected $mapping;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('uuid'),
      $container->get('paragraphs_blokkli.helper'),
      $container->get('file.repository'),
      $container->get('renderer'),
      $container->get('rokka.service'),
      $container->get('file.usage'),
      $container->get('config.factory')
    );
  }

  /**
   * Convert constructor.
   *
   * @param array $configuration
   *   The plugin configuration array.
   * @param string $pluginId
   *   The plugin id.
   * @param array $pluginDefinition
   *   The plugin definition array.
   * @param EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param UuidInterface $uuidHelper
   *   The UUID helper.
   * @param ParagraphsBlokkliHelper $paragraphsBlokkliHelper
   *   The paragraphs blokkli helper.
   * @param FileRepositoryInterface $fileRepository
   *   The file repository.
   * @param RendererInterface $renderer
   *   The renderer.
   * @param RokkaService $rokkaService
   *   The rokka service.
   * @param FileUsageInterface $fileUsage
   *   The file usage.
   * @param ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    array $pluginDefinition,
    EntityTypeManagerInterface $entityTypeManager,
    UuidInterface $uuidHelper,
    ParagraphsBlokkliHelper $paragraphsBlokkliHelper,
    protected FileRepositoryInterface $fileRepository,
    protected RendererInterface $renderer,
    protected RokkaService $rokkaService,
    protected FileUsageInterface $fileUsage,
    protected ConfigFactoryInterface $configFactory
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition, $entityTypeManager, $uuidHelper, $paragraphsBlokkliHelper);
    $this->mapping = $configFactory->get('paragraphs_blokkli.settings')->get('media_mapping');
  }

  /**
   * Executes the mutation for creating an image paragraph.
   *
   * @param ParagraphMutationContextInterface $context
   *   The mutation context.
   * @param string $data
   *   The data representing the image.
   * @param string $fileName
   *   The name of the file.
   * @param string $hostType
   *   The type of the host entity.
   * @param string $hostUuid
   *   The UUID of the host entity.
   * @param string $hostFieldName
   *   The name of the field in the host entity.
   * @param string|null $afterUuid
   *   The UUID of the paragraph after which the new paragraph should be added.
   */
  public function execute(
    ParagraphMutationContextInterface $context,
    string $data,
    string $fileName,
    string $hostType,
    string $hostUuid,
    string $hostFieldName,
    ?string $afterUuid
  ): void {

    // Make sure the mutation creates the media entity only once.
    if (empty($this->configuration['media_id'])) {
      $renderContext = new RenderContext();
      $media = $this->renderer->executeInRenderContext($renderContext, function () use ($data, $fileName) {
        $mediaStorage = $this->entityTypeManager->getStorage('media');

        $media_type = $this->entityTypeManager->getStorage('media_type')->load('image');
        $source_field_definition = $media_type->getSource()->getSourceFieldDefinition($media_type);
        $source_field = $source_field_definition->getName();

        // Get field storage settings for source field.
        $settings = $source_field_definition->getSettings();
        $scheme = $settings['uri_scheme'];
        $destination = $scheme . '://' . trim($settings['file_directory'], '/');

        // Extract the parts of the Data URI.
        if (preg_match('/data:(?P<mime>[^;]+);(?P<encoding>\w+),(?P<data>.+)$/', $data, $matches)) {
          $data = $matches['data'];
          $encoding = $matches['encoding'];

          // Currently handling base64 encoded data. Extend this if you expect other encodings.
          if ($encoding !== 'base64') {
            return NULL;
          }
          $decoded = base64_decode($data);

          if ($scheme === 'rokka://') {
            $binary_hash = sha1($decoded);

            // Based on the binary hash, we check if the image already exists in the system.
            // If we find it, we determine the file, then the media entity and then reuse it.
            $metadata_list = $this->rokkaService->loadRokkaMetadataByBinaryHash($binary_hash);
            if ($metadata_list) {
              /** @var \Drupal\rokka\Entity\RokkaMetadata $metadata */
              $metadata = reset($metadata_list);
              $file = $this->entityTypeManager->getStorage('file')->loadByProperties(['uri' => $metadata->getUri()]);

              if ($file) {
                $file = reset($file);
                $result = $this->fileUsage->listUsage($file);
                if (!empty($result)) {
                  $media = $this->findMediaInUsages($result);
                  if ($media) {
                    return $media;
                  }
                }
              }
            }
          }

          $file = $this->fileRepository->writeData($decoded, $destination . $fileName);

          $data = [
            'bundle' => 'image',
            'uuid' => $this->getUuidForNewEntity('media'),
            $source_field => [
              'target_id' => $file->id(),
            ],
          ];

          if ($scheme === 'rokka://') {
            $metadata = $this->rokkaService->loadRokkaMetadataByUri($file->getFileUri());

            if ($metadata) {
              $metadata = reset($metadata);
              if ($metadata && $staticMetadata = $metadata->getStaticMetadata()) {
                $staticMetadata = reset($staticMetadata);
                $current_language = \Drupal::languageManager()->getCurrentLanguage()->getId();
                $data['field_media_image']['alt'] = $staticMetadata['auto_description']['descriptions'][$current_language];
              }
            }
          }

          // Create media entity for the remote video.
          // @todo Try to parse the provided URL, extract the ID and try to find an
          // existing media entity that matches.
          $media = $mediaStorage->create($data);

          // The media entity must be saved at this point, because the media
          // library widget throws an error when it tries to generate a URL.
          // But this is pretty much the same behavior when using the media
          // library: Creating a new media there will also save it.
          $media->save();
          return $media;
        }
      });

      if (!$media) {
        throw new \Exception('Failed to create image media.');
      }
      $this->configuration['media_id'] = $media->id();

      // Remove the base64 data from the configuration and context so that it
      // doesn't get persisted in the database.
      $this->configuration['data'] = '';
      $this->setContextValue('data', '');
    }

    $paragraphBundle = !empty($this->mapping['image']['paragraph_bundle']) ? $this->mapping['image']['paragraph_bundle'] : 'image';
    $paragraphField = !empty($this->mapping['image']['paragraph_field']) ? $this->mapping['image']['paragraph_field'] : 'field_image';

    // Create the image paragraph.
    $paragraph = $this->entityTypeManager->getStorage('paragraph')->create([
      'type' => $paragraphBundle,
      'uuid' => $this->getUuidForNewEntity('paragraph'),
      $paragraphField => [
        [
          'target_id' => $this->configuration['media_id'],
        ],
      ],
    ]);
    $proxy = new ParagraphProxy($paragraph, $hostType, $hostUuid, $hostFieldName);
    $context->addProxy($proxy, $afterUuid);
  }

  private function findMediaInUsages($result) {
    foreach ($result as $usages) {
      foreach ($usages as $usage) {
        if (is_array($usage)) {
          $media_id = key($usage);
          $media = $this->entityTypeManager->getStorage('media')->load($media_id);
          if ($media) {
            return $media;
          }
        }
      }
    }

    return NULL;
  }

}
