<?php

namespace Drupal\paragraphs_blokkli\Plugin\ParagraphsBlokkli\Mutation;

use Drupal\Component\Uuid\UuidInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Render\RenderContext;
use Drupal\Core\Render\RendererInterface;
use Drupal\paragraphs_blokkli\ParagraphMutationContextInterface;
use Drupal\paragraphs_blokkli\ParagraphMutationPluginBase;
use Drupal\paragraphs_blokkli\ParagraphProxy;
use Drupal\paragraphs_blokkli\ParagraphsBlokkliHelper;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Adds a new remote video paragraph.
 *
 * @ParagraphMutation(
 *   id = "add_video_remote",
 *   label = @Translation("Add Video (Remote)"),
 *   description = @Translation("Adds a new paragraph of type Video (Remote)."),
 *   arguments = {
 *     "url" = @ContextDefinition("string",
 *       label = @Translation("The video URL."),
 *     ),
 *     "hostType" = @ContextDefinition("string",
 *       label = @Translation("The entity type of the target host.")
 *     ),
 *     "hostUuid" = @ContextDefinition("string",
 *       label = @Translation("The UUID of the target host.")
 *     ),
 *     "hostFieldName" = @ContextDefinition("string",
 *       label = @Translation("The field name of the target host.")
 *     ),
 *     "afterUuid" = @ContextDefinition("string",
 *       label = @Translation("The UUID of the paragraph after which to add this one."),
 *       required = FALSE,
 *     ),
 *   }
 * )
 */
class AddVideoRemote extends ParagraphMutationPluginBase {

  /**
   * The field mapping.
   *
   * @var array
   */
  protected $mapping;

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_type.manager'),
      $container->get('uuid'),
      $container->get('paragraphs_blokkli.helper'),
      $container->get('renderer'),
      $container->get('config.factory')
    );
  }

  /**
   * Convert constructor.
   *
   * @param array $configuration
   *   The plugin configuration array.
   * @param string $pluginId
   *   The plugin id.
   * @param array $pluginDefinition
   *   The plugin definition array.
   * @param EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager.
   * @param UuidInterface $uuidHelper
   *   The UUID helper.
   * @param ParagraphsBlokkliHelper $paragraphsBlokkliHelper
   *   The paragraphs blokkli helper.
   * @param RendererInterface $renderer
   *   The renderer.
   * @param ConfigFactoryInterface $configFactory
   *   The config factory.
   */
  public function __construct(
    array $configuration,
    $pluginId,
    array $pluginDefinition,
    EntityTypeManagerInterface $entityTypeManager,
    UuidInterface $uuidHelper,
    ParagraphsBlokkliHelper $paragraphsBlokkliHelper,
    protected RendererInterface $renderer,
    protected ConfigFactoryInterface $configFactory
  ) {
    parent::__construct($configuration, $pluginId, $pluginDefinition, $entityTypeManager, $uuidHelper, $paragraphsBlokkliHelper);
    $this->mapping = $configFactory->get('paragraphs_blokkli.settings')->get('media_mapping');
  }

  /**
   * Executes the mutation operation.
   *
   * @param ParagraphMutationContextInterface $context
   *   The mutation context.
   * @param string $url
   *   The URL of the remote video.
   * @param string $hostType
   *   The type of the host entity.
   * @param string $hostUuid
   *   The UUID of the host entity.
   * @param string $hostFieldName
   *   The field name of the host entity.
   * @param string|null $afterUuid
   *   (optional) The UUID of the entity after which the new entity should be placed.
   */
  public function execute(
    ParagraphMutationContextInterface $context,
    string $url,
    string $hostType,
    string $hostUuid,
    string $hostFieldName,
    ?string $afterUuid
  ): void {

    // Make sure the mutation creates the media entity only once.
    if (empty($this->configuration['media_id'])) {
      $renderContext = new RenderContext();
      $media = $this->renderer->executeInRenderContext($renderContext, function () use ($url) {
        $mediaStorage = $this->entityTypeManager->getStorage('media');

        $media_type = $this->entityTypeManager->getStorage('media_type')->load('remote_video');
        $source_field = $media_type->getSource()->getSourceFieldDefinition($media_type)->getName();

        // Create media entity for the remote video.
        // @todo Try to parse the provided URL, extract the ID and try to find an
        // existing media entity that matches.
        $media = $mediaStorage->create([
          'bundle' => 'remote_video',
          'uuid' => $this->getUuidForNewEntity('media'),
          $source_field => $url,
        ]);

        // The media entity must be saved at this point, because the media
        // library widget throws an error when it tries to generate a URL.
        // But this is pretty much the same behavior when using the media
        // library: Creating a new media there will also save it.
        $media->save();
        return $media;
      });
      $this->configuration['media_id'] = $media->id();
    }

    $paragraphStorage = $this->entityTypeManager->getStorage('paragraph');
    $paragraphBundle = !empty($this->mapping['remote_video']['paragraph_bundle']) ? $this->mapping['remote_video']['paragraph_bundle'] : 'video';
    $paragraphField = !empty($this->mapping['remote_video']['paragraph_field']) ? $this->mapping['remote_video']['paragraph_field'] : 'field_video';

    $paragraph = $paragraphStorage->create([
      'type' => $paragraphBundle,
      'uuid' => $this->getUuidForNewEntity('paragraph'),
      $paragraphField => [
        [
          'target_id' => $this->configuration['media_id'],
        ],
      ],
    ]);
    $proxy = new ParagraphProxy($paragraph, $hostType, $hostUuid, $hostFieldName);
    $context->addProxy($proxy, $afterUuid);
  }

}
