<?php

namespace Drupal\paragraphs_blokkli\Plugin\paragraphs\Behavior;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Entity\Display\EntityViewDisplayInterface;
use Drupal\Core\Entity\EntityFieldManagerInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\paragraphs\Entity\Paragraph;
use Drupal\paragraphs\ParagraphInterface;
use Drupal\paragraphs\ParagraphsBehaviorBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides a paragraphs behaviour plugin to store editor data.
 *
 * @ParagraphsBehavior(
 *   id = "paragraphs_blokkli_data",
 *   label = @Translation("Paragraph Blokkli Data"),
 *   description = @Translation("Stores the data from the blokkli editor."),
 * )
 */
class ParagraphsBlokkliData extends ParagraphsBehaviorBase {

  public function __construct(
    array $configuration,
    $plugin_id,
    $plugin_definition,
    EntityFieldManagerInterface $entity_field_manager,
    public ConfigFactoryInterface $configFactory,
  ) {
    parent::__construct($configuration, $plugin_id, $plugin_definition, $entity_field_manager);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('entity_field.manager'),
      $container->get('config.factory'),
    );
  }

  /**
   * {@inheritdoc}
   */
  public function view(array &$build, Paragraph $paragraph, EntityViewDisplayInterface $display, $view_mode) {
  }

  /**
   * {@inheritdoc}
   */
  public function buildBehaviorForm(ParagraphInterface $paragraph, array &$form, FormStateInterface $form_state) {

    $file_path = $this->configFactory->get('paragraphs_blokkli.settings')->get('schema_file');

    if (file_exists($file_path)) {
      $json = file_get_contents($file_path);
      $schema = json_decode($json, TRUE);

      $paragraph_bundle = $paragraph->bundle();

      if (is_array($schema) && count($schema) && !empty($schema[$paragraph_bundle])) {
        foreach ($schema[$paragraph_bundle] as $name => $option) {
          if ($type = $this->mapType($option['type'])) {

            switch ($type) {
              case 'checkbox':
              case 'textfield':
                $form[$name] = [
                  '#type' => $type,
                  // phpcs:ignore
                  '#title' => $this->t($option['label']),
                  '#default_value' => $paragraph->getBehaviorSetting($this->getPluginId(), $name, $option['default']),
                ];
                break;

              case 'checkboxes':
                $form[$name] = [
                  '#type' => $type,
                  // phpcs:ignore
                  '#title' => $this->t($option['label']),
                  // @todo Adapt schema.json to return an array.
                  '#default_value' => $paragraph->getBehaviorSetting($this->getPluginId(), $name, explode(',', $option['default'])),
                  '#options' => $option['options'],
                ];
                break;

              case 'radios':
                $form[$name] = [
                  '#type' => $type,
                  // phpcs:ignore
                  '#title' => $this->t($option['label']),
                  '#default_value' => $paragraph->getBehaviorSetting($this->getPluginId(), $name, $option['default']),
                  '#options' => $this->mapOptions($option['options']),
                ];
                break;
            }

          }
        }
      }
    }
    return $form;
  }

  /**
   * Build options.
   *
   * @param array $options
   *   The blökkli options.
   *
   * @return array
   *   The options mapped to key => label.
   */
  protected function mapOptions(array $options = []) {
    $mapped = [];

    foreach ($options as $key => $option) {
      $label = $this->mapOptionLabel($option) ?? $key;
      $mapped[$key] = $label;
    }

    return $mapped;
  }

  /**
   * Map the option label.
   *
   * @param array|string $option
   *   The option.
   *
   * @return string|null
   *   The label.
   */
  protected function mapOptionLabel($option) {
    if (is_string($option)) {
      return $option;
    }
    elseif (is_array($option)) {
      return $option['label'] ?? NULL;
    }
  }

  /**
   * {@inheritdoc}
   */
  public function settingsSummary(Paragraph $paragraph) {
    return [];
  }

  /**
   * {@inheritdoc}
   */
  public function submitBehaviorForm(ParagraphInterface $paragraph, array &$form, FormStateInterface $form_state) {
    $filtered_values = $this->filterBehaviorFormSubmitValues($paragraph, $form, $form_state);

    $paragraph->setBehaviorSettings($this->getPluginId(), $filtered_values);
  }

  private function mapType(string $input_type): string|null {
    $map = [
      'checkbox' => 'checkbox',
      'checkboxes' => 'checkboxes',
      'radios' => 'radios',
      'text' => 'textfield',
    ];
    return $map[$input_type] ?? NULL;
  }

}
